@@ -48,12 +48,17 @@ module Codec.CBOR.Magic
 
   -- int*ToInt conversions are missing because they are not needed.
 
-  , word8ToInt        -- :: Int8  -> Int
-  , word16ToInt       -- :: Int16 -> Int
-  , word32ToInt       -- :: Int32 -> Int
-  , word64ToInt       -- :: Int64 -> Int
+  , word8ToInt        -- :: Word8  -> Int
+  , word16ToInt       -- :: Word16 -> Int
+  , word32ToInt       -- :: Word32 -> Int
+  , word64ToInt       -- :: Word64 -> Int
+
+  , intToWord         -- :: Int    -> Word
+  , intToInt64        -- :: Int    -> Int64
+
+  , intToWord64       -- :: Int    -> Word64
+  , int64ToWord64     -- :: Int64  -> Word64
 
-  , intToInt64        -- :: Int   -> Int64
 #if defined(ARCH_32bit)
   , word8ToInt64      -- :: Word8  -> Int64
   , word16ToInt64     -- :: Word16 -> Int64
@@ -93,8 +98,12 @@ import           GHC.Float (castWord32ToFloat, castWord64ToDouble)
 import           Foreign.Ptr
 
 #if defined(OPTIMIZE_GMP)
+#if defined(HAVE_GHC_BIGNUM)
+import qualified GHC.Num.Integer                as BigNum
+#else
 import qualified GHC.Integer.GMP.Internals      as Gmp
 #endif
+#endif
 
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString          as BS
@@ -150,12 +159,17 @@ grabWord8 (Ptr ip#) = W8# (indexWord8OffAddr# ip# 0#)
 -- On x86 machines with GHC 7.10, we have byteswap primitives
 -- available to make this conversion very fast.
 
+#if MIN_VERSION_ghc_prim(0,8,0)
+grabWord16 (Ptr ip#) = W16# (wordToWord16# (byteSwap16# (word16ToWord# (indexWord16OffAddr# ip# 0#))))
+grabWord32 (Ptr ip#) = W32# (wordToWord32# (byteSwap32# (word32ToWord# (indexWord32OffAddr# ip# 0#))))
+#else
 grabWord16 (Ptr ip#) = W16# (narrow16Word# (byteSwap16# (indexWord16OffAddr# ip# 0#)))
 grabWord32 (Ptr ip#) = W32# (narrow32Word# (byteSwap32# (indexWord32OffAddr# ip# 0#)))
+#endif
 #if defined(ARCH_64bit)
 grabWord64 (Ptr ip#) = W64# (byteSwap# (indexWord64OffAddr# ip# 0#))
 #else
-grabWord64 (Ptr ip#) = W64# (byteSwap64# (indexWord64OffAddr# ip# 0#))
+grabWord64 (Ptr ip#) = W64# (byteSwap64# (word64ToWord# (indexWord64OffAddr# ip# 0#)))
 #endif
 
 #elif defined(MEM_UNALIGNED_OPS) && \
@@ -177,8 +191,14 @@ grabWord16 (Ptr ip#) =
     case indexWord8OffAddr# ip# 0# of
      w0# ->
       case indexWord8OffAddr# ip# 1# of
-       w1# -> W16# w0# `unsafeShiftL` 8 .|.
-              W16# w1#
+       w1# -> w16 w0# `unsafeShiftL` 8 .|.
+              w16 w1#
+  where
+#if MIN_VERSION_ghc_prim(0,8,0)
+    w16 w# = W16# (wordToWord16# (word8ToWord# w#))
+#else 
+    w16 w# = W16# w#  
+#endif
 
 grabWord32 (Ptr ip#) =
     case indexWord8OffAddr# ip# 0# of
@@ -188,10 +208,16 @@ grabWord32 (Ptr ip#) =
         case indexWord8OffAddr# ip# 2# of
          w2# ->
           case indexWord8OffAddr# ip# 3# of
-           w3# -> W32# w0# `unsafeShiftL` 24 .|.
-                  W32# w1# `unsafeShiftL` 16 .|.
-                  W32# w2# `unsafeShiftL`  8 .|.
-                  W32# w3#
+           w3# -> w32 w0# `unsafeShiftL` 24 .|.
+                  w32 w1# `unsafeShiftL` 16 .|.
+                  w32 w2# `unsafeShiftL`  8 .|.
+                  w32 w3#
+  where 
+#if MIN_VERSION_ghc_prim(0,8,0)
+    w32 w# = W32# (wordToWord32# (word8ToWord# w#))
+#else 
+    w32 w# = W32# w#  
+#endif
 
 grabWord64 (Ptr ip#) =
     case indexWord8OffAddr# ip# 0# of
@@ -209,19 +235,27 @@ grabWord64 (Ptr ip#) =
                 case indexWord8OffAddr# ip# 6# of
                  w6# ->
                   case indexWord8OffAddr# ip# 7# of
-                   w7# -> w w0# `unsafeShiftL` 56 .|.
-                          w w1# `unsafeShiftL` 48 .|.
-                          w w2# `unsafeShiftL` 40 .|.
-                          w w3# `unsafeShiftL` 32 .|.
-                          w w4# `unsafeShiftL` 24 .|.
-                          w w5# `unsafeShiftL` 16 .|.
-                          w w6# `unsafeShiftL`  8 .|.
-                          w w7#
+                   w7# -> w64 w0# `unsafeShiftL` 56 .|.
+                          w64 w1# `unsafeShiftL` 48 .|.
+                          w64 w2# `unsafeShiftL` 40 .|.
+                          w64 w3# `unsafeShiftL` 32 .|.
+                          w64 w4# `unsafeShiftL` 24 .|.
+                          w64 w5# `unsafeShiftL` 16 .|.
+                          w64 w6# `unsafeShiftL`  8 .|.
+                          w64 w7#
   where
-#if defined(ARCH_64bit)
-    w w# = W64# w#
-#else
-    w w# = W64# (wordToWord64# w#)
+#if MIN_VERSION_ghc_prim(0,8,0)
+    toWord :: Word8# -> Word#
+    toWord w# = word8ToWord# w#
+#else 
+    toWord :: Word# -> Word#
+    toWord w# = w#  
+#endif
+
+#if WORD_SIZE_IN_BITS == 64
+    w64 w# = W64# (toWord w#)
+#else 
+    w64 w# = W64# (wordToWord64# (toWord w#))
 #endif
 
 #endif
@@ -388,6 +422,31 @@ intToInt64 :: Int -> Int64
 intToInt64 = fromIntegral
 {-# INLINE intToInt64 #-}
 
+intToWord :: Int -> Word
+intToWord = fromIntegral
+{-# INLINE intToWord #-}
+
+intToWord64 :: Int -> Word64
+intToWord64 = fromIntegral
+{-# INLINE intToWord64 #-}
+
+int64ToWord64 :: Int64 -> Word64
+int64ToWord64 = fromIntegral
+{-# INLINE int64ToWord64 #-}
+
+#if MIN_VERSION_ghc_prim(0,8,0)
+word8ToWord  (W8#  w#) = W# (word8ToWord# w#)
+word16ToWord (W16# w#) = W# (word16ToWord# w#)
+word32ToWord (W32# w#) = W# (word32ToWord# w#)
+#if defined(ARCH_64bit)
+word64ToWord (W64# w#) = W# w#
+#else
+word64ToWord (W64# w64#) =
+  case isTrue# (w64# `leWord64#` wordToWord64# 0xffffffff##) of
+    True  -> Just (W# (word64ToWord# w64#))
+    False -> Nothing
+#endif
+#else
 word8ToWord  (W8#  w#) = W# w#
 word16ToWord (W16# w#) = W# w#
 word32ToWord (W32# w#) = W# w#
@@ -399,12 +458,25 @@ word64ToWord (W64# w64#) =
     True  -> Just (W# (word64ToWord# w64#))
     False -> Nothing
 #endif
+#endif
 
 {-# INLINE word8ToWord #-}
 {-# INLINE word16ToWord #-}
 {-# INLINE word32ToWord #-}
 {-# INLINE word64ToWord #-}
 
+#if MIN_VERSION_ghc_prim(0,8,0)
+word8ToInt  (W8#  w#) = I# (word2Int# (word8ToWord# w#))
+word16ToInt (W16# w#) = I# (word2Int# (word16ToWord# w#))
+#if defined(ARCH_64bit)
+word32ToInt (W32# w#) = I# (word2Int# (word32ToWord# w#))
+#else
+word32ToInt (W32# w#) =
+  case isTrue# (w# `ltWord#` 0x80000000##) of
+    True  -> Just (I# (word2Int# (word32ToWord# w#)))
+    False -> Nothing
+#endif
+#else
 word8ToInt  (W8#  w#) = I# (word2Int# w#)
 word16ToInt (W16# w#) = I# (word2Int# w#)
 
@@ -416,6 +488,7 @@ word32ToInt (W32# w#) =
     True  -> Just (I# (word2Int# w#))
     False -> Nothing
 #endif
+#endif
 
 #if defined(ARCH_64bit)
 word64ToInt (W64# w#) =
@@ -477,7 +550,12 @@ uintegerFromBytes (BS.PS fp (I# off#) (I# len#)) =
           let addrOff# = addr# `plusAddr#` off#
           -- The last parmaeter (`1#`) tells the import function to use big
           -- endian encoding.
-          in Gmp.importIntegerFromAddr addrOff# (int2Word# len#) 1#
+          in
+#if defined(HAVE_GHC_BIGNUM)
+             BigNum.integerFromAddr (int2Word# len#) addrOff# 1#
+#else
+             Gmp.importIntegerFromAddr addrOff# (int2Word# len#) 1#
+#endif
 #else
 uintegerFromBytes bs =
     case BS.uncons bs of
